#!/usr/bin/env phantomjs

var CONSOLE_USERNAME = 'admin@foo.bar',
    CONSOLE_PASSWORD = 'console_auth';

if (phantom.args.length === 0) {
  console.log('Usage: run_jasmine_test.js <url>');

  phantom.exit(1);
}

// Function.bind polyfill
if (!Function.prototype.bind) {
  Function.prototype.bind = function (oThis) {
    if (typeof this !== "function") {
      // closest thing possible to the ECMAScript 5 internal IsCallable function
      throw new TypeError("Function.prototype.bind - what is trying to be bound is not callable");
    }

    var aArgs = Array.prototype.slice.call(arguments, 1),
      fToBind = this,
      fNOP = function () {},
      fBound = function () {
        return fToBind.apply(this instanceof fNOP && oThis
          ? this
          : oThis,
          aArgs.concat(Array.prototype.slice.call(arguments)));
      };

    fNOP.prototype = this.prototype;
    fBound.prototype = new fNOP();

    return fBound;
  };
}

function PhantomJasmineRunner(page) {

  this.state = 'LOGIN'; // LOGIN or LOGGEDIN

  this.page = page;

  this.page.onConsoleMessage = this.check_console.bind(this);

  this.page.customHeaders = {
    'Accept-Language': 'en-us,en;q=0.5' // required due to rubycas not accepting the default language string(!?)
  };
}

PhantomJasmineRunner.prototype = {

  run: function(address) {
    this.page.open(address, this.page_open.bind(this));
  },

  page_open: function(status) {

    // note that 'success' is sent for 500's and 404's
    if (status !== 'success') {
      console.log("Can't load URL: " + address);

      phantom.exit(1);
    }

    switch (this.state) {
      case 'LOGIN':

        this.login();
        return;

      default:

        // You can't just get the HTTP status code of the request, so do a basic check
        // to make sure we've loaded a unit test resource.
        if (!this.jasmine_exists()) {
          console.log('CONSOLE_REPORTER global not present, the URL provided was not a Jasmine test harness.');

          phantom.exit(1);
        }

        return;
    }
  },

  login: function() {
    if (!this.login_form_exists()) {
      console.log('Could not log in to console_auth - was the correct URL provided?');

      phantom.exit(1);
    }

    var login = this.page.evaluate(function(username, password) {
      try {
        var form = document.getElementById('login'),
            username_el = document.getElementById('username'),
            password_el = document.getElementById('password');

        username_el.value = username;
        password_el.value = password;

        form.submit();

        return true;
      } catch (e) {
        console.log('Error trying to log in to console_auth: ' + e);
        return false;
      }
    }, CONSOLE_USERNAME, CONSOLE_PASSWORD);

    if (!login) {
      console.log('Could not log in to the console_auth login form - has the structure changed?');

      phantom.exit(1);
    }

    this.state = 'LOGGEDIN';
  },

  check_console: function(message) {
    console.log(message);

    if (this.state == 'LOGGEDIN' && message === 'ConsoleReporter finished') {
      this.terminate();
    }
  },

  get_status: function() {
    this.page.evaluate(function() {
      return CONSOLE_REPORTER.status;
    });
  },

  terminate: function() {
    switch (this.get_status()) {
      case 'success':
        return phantom.exit(0);
      case 'fail':
        return phantom.exit(1);
      default:
        return phantom.exit(2);
    }
  },

  jasmine_exists: function() {
    return this.page.evaluate(function() {
              return typeof CONSOLE_REPORTER !== 'undefined';
            });
  },

  login_form_exists: function() {
    return this.page.evaluate(function() {
              return document.getElementById('login');
            });
  }
};

var page = new WebPage(),
    runner = new PhantomJasmineRunner(page),
    address = phantom.args[0];

runner.run(address);
